use std::thread;
use std::sync::atomic::{AtomicUsize, Ordering};
use std::sync::Arc;

fn f() {
    let data = vec![1, 2, 3, 4];
    let idx = Arc::new(AtomicUsize::new(0)); //pointer 1
    let other_idx = idx.clone();                      //pointer 2

// `move` captures other_idx by-value, moving it into this thread
    thread::spawn(move || {
        // It's ok to mutate idx because this value
        // is an atomic, so it can't cause a Data Race.
        other_idx.fetch_add(10, Ordering::SeqCst); //*idx = *idx + 10
    });

    if idx.load(Ordering::SeqCst) < data.len() { //*idx < data.length
        unsafe {
            // Incorrectly loading the idx after we did the bounds check.
            // It could have changed. This is a race condition, *and dangerous*
            // because we decided to do `get_unchecked`, which is `unsafe`.
            println!("{}", data.get_unchecked(idx.load(Ordering::SeqCst))); //data[*idx]
        }
    }
}
